/*--------------------------- fft1.c ---------------------------------- */
/*									*/
/* Author:	Eyal Lebedinsky (eyal@ise.canberra.edu.au)		*/
/* Date:	Jan 1994						*/
/* Version:	22 January 1994						*/
/*									*/
/* Measure the fft() time. NOTE that on UNIX and other multiprogramming	*/
/* systems you will need to replace time() with a cpu timing function.	*/
/*									*/
/*--------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

extern unsigned short FAR get_timer (void);
extern void FAR set_timer (void);
extern unsigned long FAR timer_micros (unsigned short old, unsigned short new);
extern void FAR fft ();
extern void FAR dummy ();

extern void FAR
dummy ()
{
	return;
}

#define	LN	12
#define N	(1<<LN)

short		NEAR x[N] = {0};
short		NEAR qf[N/2+1] = {0};
static short	xx[N];

static void NEAR
test_once (f, repeat)
void		(FAR *f)();
unsigned long	repeat;
{
	long	n;

	for (n = repeat; n > 0; --n) {
		memcpy (x, xx, sizeof (x));
		(*f)();
	}
}

static unsigned long NEAR
test_all (f, repeat, retest)
void		(FAR *f)();
unsigned long	repeat;
unsigned long	retest;
{
	unsigned short	t;
	unsigned long	tnow, tlow;

	tlow = 0xffffffffUL;
	while (retest-- > 0) {
		set_timer();
		t = get_timer();
		test_once (f, repeat);
		tnow = timer_micros (t, get_timer());
		if (tlow > tnow)
			tlow = tnow;
	}
	return (tlow);
}

static void
dump_results (void)
{
	int	i;

	for (i = 0; i <= N/2; ++i)
		printf ("%4u %5u\n", i, qf[i]);

}

int
main(int argc, char *argv[])
{
	int		i;
	unsigned long	t1, t2;
	unsigned long	l;
	unsigned long	repeat = REPEAT;
	unsigned long	retest = RETEST;
	int		dump = 0;
	int		method = 1;

        for (i = 1; i < argc; ++i) {
		if ('n' == argv[i][0]) {
			if (1 != sscanf (argv[i]+1, "%lu", &l)
							     || l <= 0)
				printf ("bad repeat '%s' ignored\n", argv[i]);
			else
				repeat = l;
		} else if ('r' == argv[i][0]) {
			if (1 != sscanf (argv[i]+1, "%lu", &l)
							     || l <= 0)
				printf ("bad retest '%s' ignored\n", argv[i]);
			else
				retest = l;
		} else if ('m' == argv[i][0]) {
			if (1 != sscanf (argv[i]+1, "%lu", &l)
							     || l <= 0)
				printf ("bad method '%s' ignored\n", argv[i]);
			else
				method = l;
		} else if ('d' == argv[i][0])
			dump = 1;
		else
			printf ("unknowd option '%s' ignored\n", argv[i]);
	}

	srand (0);
	for (i = 0; i < N; ++i) {
		switch (method) {
		default:
		case 1:
			xx[i] = (short)((rand () * (long)rand ()) >> 8);
			break;
		case 2:
			xx[i] = (i & (N/16)) ? 0x8000 : 0x7fff;
			break;
		case 3:
			xx[i] = i << (3 + 16 - LN);
			break;
		}
	}

	t1 = test_all (fft, repeat, retest);
	t2 = test_all (dummy, repeat, retest);
	printf ("fft1> fft called %lu times\n", repeat);
	printf ("fft1> test repeated %lu times\n", retest);
	printf ("fft1> test sample method %d\n", method);
	printf ("fft1> real =%lu\n", t1);
	printf ("fft1> dummy=%lu\n", t2);
	printf ("fft1> nett =%lu\n", t1 -= t2);

	t1 /= repeat;
	printf ("fft1> milliseconds per fft(): %lu.%03lu\n",
		t1/1000, t1%1000);

	if (dump)
		dump_results ();

	exit (0);
}
